@@ -22,15 +22,14 @@ import json
 import logging
 import os
 import os.path as op
-import re
 import traceback
+import warnings
 from abc import ABCMeta, abstractmethod
-from typing import List
+from typing import Any, Dict, Iterable, Optional
 
 from splunklib import binding
 
-from .. import splunk_rest_client as rest_client
-from ..utils import retry
+from solnlib import _utils, utils
 
 __all__ = ["CheckpointerException", "KVStoreCheckpointer", "FileCheckpointer"]
 
@@ -43,85 +42,21 @@ class Checkpointer(metaclass=ABCMeta):
     """Base class of checkpointer."""
 
     @abstractmethod
-    def update(self, key: str, state: dict):
-        """Update checkpoint.
-
-        Arguments:
-            key: Checkpoint key.
-            state: Checkpoint state.
-
-        Examples:
-           >>> from solnlib.modular_input import checkpointer
-           >>> ck = checkpointer.KVStoreCheckpointer(session_key,
-                                                     'Splunk_TA_test')
-           >>> ck.update('checkpoint_name1', {'k1': 'v1', 'k2': 'v2'})
-           >>> ck.update('checkpoint_name2', 'checkpoint_value2')
-        """
-
-        pass
+    def update(self, key: str, state: Any):
+        """Updates document with an id that equals to `key` and `state` as
+        document data."""
 
     @abstractmethod
-    def batch_update(self, states: List):
-        """Batch update checkpoint.
-
-        Arguments:
-            states: List of checkpoint. Each state in the list is a
-                json object which should contain '_key' and 'state' keys.
-                For instance::
-
-                    {
-                        '_key': ckpt key which is a string,
-                        'state': ckpt which is a json object
-                    }
-
-        Examples:
-           >>> from solnlib.modular_input import checkpointer
-           >>> ck = checkpointer.KVStoreCheckpointer(session_key,
-                                                     'Splunk_TA_test')
-           >>> ck.batch_update([{'_key': 'checkpoint_name1',
-                                 'state': {'k1': 'v1', 'k2': 'v2'}},
-                                {'_key': 'checkpoint_name2',
-                                 'state': 'checkpoint_value2'},
-                                {...}])
-        """
-
-        pass
+    def batch_update(self, states: Iterable[Dict[str, Any]]):
+        """Updates multiple documents."""
 
     @abstractmethod
     def get(self, key: str) -> dict:
-        """Get checkpoint.
-
-        Arguments:
-            key: Checkpoint key.
-
-        Returns:
-            Checkpoint state if exists else None.
-
-        Examples:
-           >>> from solnlib.modular_input import checkpointer
-           >>> ck = checkpointer.KVStoreCheckpointer(session_key,
-                                                     'Splunk_TA_test')
-           >>> ck.get('checkpoint_name1')
-           >>> returns: {'k1': 'v1', 'k2': 'v2'}
-        """
-
-        pass
+        """Gets document with an id that equals to `key`."""
 
     @abstractmethod
     def delete(self, key: str):
-        """Delete checkpoint.
-
-        Arguments:
-            key: Checkpoint key.
-
-        Examples:
-           >>> from solnlib.modular_input import checkpointer
-           >>> ck = checkpointer.KVStoreCheckpointer(session_key,
-                                                     'Splunk_TA_test')
-           >>> ck.delete('checkpoint_name1')
-        """
-
-        pass
+        """Deletes document with an id that equals to `key`."""
 
 
 class KVStoreCheckpointer(Checkpointer):
@@ -129,13 +64,19 @@ class KVStoreCheckpointer(Checkpointer):
 
     Use KVStore to save modular input checkpoint.
 
+    More information about KV Store in Splunk is
+    [here](https://dev.splunk.com/enterprise/docs/developapps/manageknowledge/kvstore/aboutkvstorecollections).
+
     Examples:
         >>> from solnlib.modular_input import checkpointer
-        >>> ck = checkpoint.KVStoreCheckpointer('TestKVStoreCheckpointer',
-                                                session_key,
-                                                'Splunk_TA_test')
-        >>> ck.update(...)
-        >>> ck.get(...)
+        >>> checkpoint = checkpointer.KVStoreCheckpointer(
+                "unique_addon_checkpoints",
+                "session_key",
+                "unique_addon"
+            )
+        >>> checkpoint.update("input_1", {"timestamp": 1638043093})
+        >>> checkpoint.get("input_1")
+        >>> # returns {"timestamp": 1638043093}
     """
 
     def __init__(
@@ -143,11 +84,11 @@ class KVStoreCheckpointer(Checkpointer):
         collection_name: str,
         session_key: str,
         app: str,
-        owner: str = "nobody",
-        scheme: str = None,
-        host: str = None,
-        port: int = None,
-        **context: dict
+        owner: Optional[str] = "nobody",
+        scheme: Optional[str] = None,
+        host: Optional[str] = None,
+        port: Optional[int] = None,
+        **context: Any,
     ):
         """Initializes KVStoreCheckpointer.
 
@@ -162,84 +103,103 @@ class KVStoreCheckpointer(Checkpointer):
             context: Other configurations for Splunk rest client.
 
         Raises:
-            CheckpointerException: If init kvstore checkpointer failed.
+            binding.HTTPError: HTTP error different from 404, for example 503
+                when KV Store is initializing and not ready to serve requests.
+            CheckpointerException: If init KV Store checkpointer failed.
         """
         try:
-            self._collection_data = self._get_collection_data(
-                collection_name, session_key, app, owner, scheme, host, port, **context
+            if not context.get("pool_connections"):
+                context["pool_connections"] = 5
+            if not context.get("pool_maxsize"):
+                context["pool_maxsize"] = 5
+            self._collection_data = _utils.get_collection_data(
+                collection_name,
+                session_key,
+                app,
+                owner,
+                scheme,
+                host,
+                port,
+                {"state": "string"},
+                **context,
             )
         except KeyError:
-            raise CheckpointerException("Get kvstore checkpointer failed.")
+            raise CheckpointerException("Get KV Store checkpointer failed.")
 
-    @retry(exceptions=[binding.HTTPError])
-    def _get_collection_data(
-        self, collection_name, session_key, app, owner, scheme, host, port, **context
-    ):
+    @utils.retry(exceptions=[binding.HTTPError])
+    def update(self, key: str, state: Any) -> None:
+        """Updates document with an id that equals to `key` and `state` as
+        document data.
 
-        if not context.get("pool_connections"):
-            context["pool_connections"] = 5
+        Arguments:
+            key: `id` of the document to update.
+            state: Document data to update. It can be integer, string,
+                or a dict, or anything that can be an argument to `json.dumps`.
 
-        if not context.get("pool_maxsize"):
-            context["pool_maxsize"] = 5
-
-        kvstore = rest_client.SplunkRestClient(
-            session_key,
-            app,
-            owner=owner,
-            scheme=scheme,
-            host=host,
-            port=port,
-            **context
-        ).kvstore
-
-        collection_name = re.sub(r"[^\w]+", "_", collection_name)
-        try:
-            kvstore.get(name=collection_name)
-        except binding.HTTPError as e:
-            if e.status != 404:
-                raise
-
-            fields = {"state": "string"}
-            kvstore.create(collection_name, fields=fields)
-
-        collections = kvstore.list(search=collection_name)
-        for collection in collections:
-            if collection.name == collection_name:
-                return collection.data
-        else:
-            raise KeyError("Get collection data: %s failed." % collection_name)
-
-    @retry(exceptions=[binding.HTTPError])
-    def update(self, key, state):
+        Raises:
+            binding.HTTPError: when an error occurred in Splunk, for example,
+                when Splunk is restarting and KV Store is not yet initialized.
+        """
         record = {"_key": key, "state": json.dumps(state)}
         self._collection_data.batch_save(record)
 
-    @retry(exceptions=[binding.HTTPError])
-    def batch_update(self, states):
+    @utils.retry(exceptions=[binding.HTTPError])
+    def batch_update(self, states: Iterable[Dict[str, Any]]) -> None:
+        """Updates multiple documents.
+
+        Arguments:
+            states: Iterable that contains documents to update. Document should
+                be a dict with at least "state" key.
+
+        Raises:
+            binding.HTTPError: when an error occurred in Splunk, for example,
+                when Splunk is restarting and KV Store is not yet initialized.
+        """
         for state in states:
             state["state"] = json.dumps(state["state"])
         self._collection_data.batch_save(*states)
 
-    @retry(exceptions=[binding.HTTPError])
-    def get(self, key):
+    @utils.retry(exceptions=[binding.HTTPError])
+    def get(self, key: str) -> Optional[Any]:
+        """Gets document with an id that equals to `key`.
+
+        Arguments:
+            key: `id` of the document to get.
+
+        Raises:
+            binding.HTTPError: When an error occurred in Splunk (not 404 code),
+                can be 503 code, when Splunk is restarting and KV Store is not
+                yet initialized.
+
+        Returns:
+            Document data under `key` or `None` in case of no data.
+        """
         try:
             record = self._collection_data.query_by_id(key)
         except binding.HTTPError as e:
             if e.status != 404:
-                logging.error("Get checkpoint failed: %s.", traceback.format_exc())
+                logging.error(f"Get checkpoint failed: {traceback.format_exc()}.")
                 raise
-
             return None
-
         return json.loads(record["state"])
 
-    @retry(exceptions=[binding.HTTPError])
-    def delete(self, key):
+    @utils.retry(exceptions=[binding.HTTPError])
+    def delete(self, key: str) -> None:
+        """Deletes document with an id that equals to `key`.
+
+        Arguments:
+            key: `id` of the document to delete.
+
+        Raises:
+            binding.HTTPError: When an error occurred in Splunk (not 404 code),
+                can be 503 code, when Splunk is restarting and KV Store is not
+                yet initialized.
+        """
         try:
             self._collection_data.delete_by_id(key)
         except binding.HTTPError as e:
             if e.status != 404:
-                logging.error("Delete checkpoint failed: %s.", traceback.format_exc())
+                logging.error(f"Delete checkpoint failed: {traceback.format_exc()}.")
                 raise
 
 
@@ -261,6 +221,10 @@ class FileCheckpointer(Checkpointer):
         Arguments:
             checkpoint_dir: Checkpoint directory.
         """
+        warnings.warn(
+            "FileCheckpointer is deprecated, please use KVStoreCheckpointer",
+            stacklevel=2,
+        )
         self._checkpoint_dir = checkpoint_dir
 
     def encode_key(self, key):
